# Copyright (C) 2024 Atomicorp, Inc.
# All rights reserved.
#
# Version: 1.0

# Log location
$LogFile = "C:\Program Files (x86)\ossec-agent\active-response\active-responses.log"
$IsolationHosts = "C:\Program Files (x86)\ossec-agent\shared\isolation-hosts"
$Debug = $true

function Write-Log {
    param($Message)
    $DateStr = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
    "$DateStr $Message" | Out-File -FilePath $LogFile -Append
}

function Write-Debug {
    param($Message)
    if ($Debug) {
        Write-Log "DEBUG: $Message"
    }
}

function Test-Administrator {
    $user = [Security.Principal.WindowsIdentity]::GetCurrent()
    $principal = New-Object Security.Principal.WindowsPrincipal($user)
    return $principal.IsInRole([Security.Principal.WindowsBuiltInRole]::Administrator)
}

function Get-WindowsVersion {
    $os = [Environment]::OSVersion.Version
    Write-Debug "Detected Windows version: $($os.Major).$($os.Minor)"
    return $os
}

# Check for admin privileges
if (-not (Test-Administrator)) {
    Write-Log "ERROR: This script must be run as Administrator"
    exit 1
}

# Detect Windows version and set firewall mode
$WindowsVersion = Get-WindowsVersion
$UseAdvFirewall = $WindowsVersion.Major -ge 6  # Vista/2008 or later
Write-Debug "Using $(if ($UseAdvFirewall) {'advanced'} else {'legacy'}) firewall commands"

# Read and parse input
if ($args.Count -gt 0) {
    $Action = $args[0]
    Write-Debug "Running with command line args: ACTION=$Action"
} else {
    $stdin = [Console]::In.ReadLine()
    Write-Debug "Received stdin data: $stdin"
    if ($stdin -match "enable") {
        $Action = "add"
    } elseif ($stdin -match "disable") {
        $Action = "delete"
    } else {
        Write-Log "ERROR: Invalid input - could not determine action"
        exit 1
    }
    Write-Debug "Determined ACTION=$Action from stdin"
}

# Check if isolation hosts file exists
if (-not (Test-Path $IsolationHosts)) {
    Write-Log "ERROR: Isolation hosts file not found at $IsolationHosts"
    exit 1
}
Write-Debug "Found isolation hosts file at $IsolationHosts"

function Apply-Isolation-Advanced {
    Write-Debug "Starting apply_isolation function (Advanced Firewall)"
    
    try {
        Write-Debug "Resetting Windows Firewall to defaults"
        netsh advfirewall reset | Out-Null

        Write-Debug "Setting default policies to BLOCK"
        netsh advfirewall set allprofiles firewallpolicy blockinbound,blockoutbound | Out-Null

        # Allow loopback
        Write-Debug "Adding loopback rules"
        netsh advfirewall firewall add rule name="Isolation-Allow-Loopback-In" dir=in action=allow localip=127.0.0.1 | Out-Null
        netsh advfirewall firewall add rule name="Isolation-Allow-Loopback-Out" dir=out action=allow remoteip=127.0.0.1 | Out-Null

        # Allow established connections
        Write-Debug "Adding established connection rules"
        netsh advfirewall set allprofiles settings statefulftp enable | Out-Null
        netsh advfirewall set allprofiles settings inboundusernotification enable | Out-Null

        # Add rules for allowed IPs
        Get-Content $IsolationHosts | ForEach-Object {
            if ($_ -match '\S' -and -not $_.StartsWith("#")) {
                Write-Debug "Adding rules for IP: $_"
                netsh advfirewall firewall add rule name="Isolation-Allow-$_-In" dir=in action=allow remoteip=$_ | Out-Null
                netsh advfirewall firewall add rule name="Isolation-Allow-$_-Out" dir=out action=allow remoteip=$_ | Out-Null
            }
        }

        Write-Debug "Listing final firewall rules:"
        netsh advfirewall firewall show rule name=all | Out-File -FilePath $LogFile -Append
        Write-Log "Applied Windows isolation rules successfully (Advanced Firewall)"
    }
    catch {
        Write-Log "ERROR: Failed to apply firewall rules: $_"
        exit 1
    }
}

function Apply-Isolation-Legacy {
    Write-Debug "Starting apply_isolation function (Legacy Firewall)"
    
    try {
        Write-Debug "Resetting Windows Firewall to defaults"
        netsh firewall reset | Out-Null

        Write-Debug "Setting default policies to BLOCK"
        netsh firewall set opmode mode=ENABLE | Out-Null
        netsh firewall set notifications enable | Out-Null

        # Block all by default
        netsh firewall set defaultinbound mode=DISABLE | Out-Null
        netsh firewall set defaultoutbound mode=DISABLE | Out-Null

        # Allow loopback
        Write-Debug "Adding loopback rules"
        netsh firewall add allowedip address=127.0.0.1 | Out-Null

        # Add rules for allowed IPs
        Get-Content $IsolationHosts | ForEach-Object {
            if ($_ -match '\S' -and -not $_.StartsWith("#")) {
                Write-Debug "Adding rules for IP: $_"
                netsh firewall add allowedip address=$_ | Out-Null
            }
        }

        Write-Debug "Listing final firewall rules:"
        netsh firewall show config | Out-File -FilePath $LogFile -Append
        Write-Log "Applied Windows isolation rules successfully (Legacy Firewall)"
    }
    catch {
        Write-Log "ERROR: Failed to apply firewall rules: $_"
        exit 1
    }
}

function Remove-Isolation-Advanced {
    Write-Debug "Starting remove_isolation function (Advanced Firewall)"
    
    try {
        Write-Debug "Removing isolation firewall rules"
        netsh advfirewall firewall delete rule name="Isolation-Allow-*" | Out-Null
        
        Write-Debug "Resetting Windows Firewall to defaults"
        netsh advfirewall reset | Out-Null
        
        Write-Debug "Setting default policies to ALLOW"
        netsh advfirewall set allprofiles firewallpolicy allowinbound,allowoutbound | Out-Null

        Write-Debug "Listing final firewall rules:"
        netsh advfirewall firewall show rule name=all | Out-File -FilePath $LogFile -Append
        Write-Log "Removed Windows isolation rules successfully (Advanced Firewall)"
    }
    catch {
        Write-Log "ERROR: Failed to remove firewall rules: $_"
        exit 1
    }
}

function Remove-Isolation-Legacy {
    Write-Debug "Starting remove_isolation function (Legacy Firewall)"
    
    try {
        Write-Debug "Resetting Windows Firewall to defaults"
        netsh firewall reset | Out-Null
        
        Write-Debug "Setting default policies to ALLOW"
        netsh firewall set opmode mode=ENABLE | Out-Null
        netsh firewall set defaultinbound mode=ENABLE | Out-Null
        netsh firewall set defaultoutbound mode=ENABLE | Out-Null

        Write-Debug "Listing final firewall rules:"
        netsh firewall show config | Out-File -FilePath $LogFile -Append
        Write-Log "Removed Windows isolation rules successfully (Legacy Firewall)"
    }
    catch {
        Write-Log "ERROR: Failed to remove firewall rules: $_"
        exit 1
    }
}

# Main execution
Write-Debug "Starting main execution with ACTION=$Action"
switch ($Action) {
    "add" {
        Write-Log "Applying isolation rules"
        if ($UseAdvFirewall) {
            Apply-Isolation-Advanced
        } else {
            Apply-Isolation-Legacy
        }
    }
    "delete" {
        Write-Log "Removing isolation rules"
        if ($UseAdvFirewall) {
            Remove-Isolation-Advanced
        } else {
            Remove-Isolation-Legacy
        }
    }
    default {
        Write-Log "Invalid action: $Action"
        exit 1
    }
}

Write-Debug "Script completed successfully"
exit 0 